==> pge-1.0.8.2.tar.gz: pge-1.0.8.2/pge/model/growth/pa.py <==
import numpy as np


from pge.model.growth.basic import BUnGrowth, BDiGrowth
from pge.model.growth.basic_fix import FixUnGrowth, FixDiGrowth


class PAGrowth(BUnGrowth):
    def choice(self, graph, sz):
        probs = np.array([graph.count_in_degree(node) for node in graph.get_ids()]) + self.param
        probs = probs / np.sum(probs)
        return np.random.choice(graph.get_ids(), sz, replace=False, p=probs)


class PADiGrowth(BDiGrowth):
    def choice(self, graph, sz, tp="in"):
        if tp == "in":
            probs = np.array([graph.count_in_degree(node) for node in graph.get_ids()]) + self.param[0]
        else:
            probs = np.array([graph.count_out_degree(node) for node in graph.get_ids()]) + self.param[1]
        probs = probs / np.sum(probs)
        return np.random.choice(graph.get_ids(), sz, replace=False, p=probs)


class PAFixUnGrowth(FixUnGrowth):
    def choice(self, graph, sz):
        probs = graph.get_attributes("dg") + self.param
        probs = probs / np.sum(probs)
        return np.random.choice(graph.get_ids(stable=True), sz, replace=False, p=probs)

    def clean(self, graph):
        graph = self.prep(graph)
        if self.param != 0:
            return []

        dels = graph.get_ids(stable=True)[(graph.get_attributes("dg") == 0)]
        return dels

    def prep(self, graph):
        graph.set_attrs(
            "dg", {node: graph.count_in_degree(node) for node in graph.get_ids()}
        )
        return graph


class PAFixDiGrowth(FixDiGrowth):
    def choice(self, graph, sz, tp="in"):
        probs = graph.get_attributes("dg_" + tp) + self.param[tp != "in"]
        probs = probs / np.sum(probs)
        return np.random.choice(graph.get_ids(stable=True), sz, replace=False, p=probs)

    def clean(self, graph):
        if np.sum(self.param) != 0:
            return []

        graph = self.prep(graph)
        dels = graph.get_ids(stable=True)[(graph.get_attributes("dg_in") == 0)
                                         & (graph.get_attributes("dg_out") == 0)]
        return dels

    def prep(self, graph):
        graph.set_attrs(
            "dg_in", {node: graph.count_in_degree(node) for node in graph.get_ids()}
        )
        graph.set_attrs(
            "dg_out", {node: graph.count_out_degree(node) for node in graph.get_ids()}
        )
        return graph

==> pge-1.0.9.3.tar.gz: pge-1.0.9.3/pge/model/growth/pa.py <==
import numpy as np

from pge.model.growth.basic import BUnGrowth, BDiGrowth
from pge.model.growth.basic_fix import FixUnGrowth, FixDiGrowth


class PAGrowth(BUnGrowth):
    def choice(self, graph, sz):
        ids = graph.get_ids(stable=True)
        probs = np.array([graph.count_in_degree(node) for node in ids]) + self.param
        probs = probs / np.sum(probs)
        probs, ids = probs[probs > 0], ids[probs > 0]
        return np.random.choice(ids, sz, replace=False, p=probs)


class PADiGrowth(BDiGrowth):
    def choice(self, graph, sz, tp="in"):
        ids = graph.get_ids(stable=True)
        if tp == "in":
            probs = np.array([graph.count_in_degree(node) for node in ids]) + self.param[0]
        else:
            probs = np.array([graph.count_out_degree(node) for node in ids]) + self.param[1]
        probs = probs / np.sum(probs)
        probs, ids = probs[probs > 0], ids[probs > 0]
        return np.random.choice(ids, sz, replace=False, p=probs)


class PAFixUnGrowth(FixUnGrowth):
    def choice(self, graph, sz):
        probs = graph.get_attributes("dg") + self.param
        probs = probs / np.sum(probs)
        return np.random.choice(graph.get_ids(stable=True), sz, replace=False, p=probs)

    def clean(self, graph):
        graph = self.prep(graph)
        if self.param != 0:
            return []

        dels = graph.get_ids(stable=True)[(graph.get_attributes("dg") == 0)]
        return dels

    def prep(self, graph):
        graph.set_attrs(
            "dg", {node: graph.count_in_degree(node) for node in graph.get_ids()}
        )
        return graph


class PAFixDiGrowth(FixDiGrowth):
    def choice(self, graph, sz, tp="in"):
        ids = graph.get_ids(stable=True)
        probs = graph.get_attributes("dg_" + tp) + self.param[tp != "in"]
        probs = probs / np.sum(probs)
        probs, ids = probs[probs > 0], ids[probs > 0]
        return np.random.choice(ids, sz, replace=False, p=probs)

    def clean(self, graph):
        if np.sum(self.param) != 0:
            return []

        graph = self.prep(graph)
        dels = graph.get_ids(stable=True)[(graph.get_attributes("dg_in") == 0)
                                          & (graph.get_attributes("dg_out") == 0)]
        return dels

    def prep(self, graph):
        graph.set_attrs(
            "dg_in", {node: graph.count_in_degree(node) for node in graph.get_ids()}
        )
        graph.set_attrs(
            "dg_out", {node: graph.count_out_degree(node) for node in graph.get_ids()}
        )
        return graph
